import 'package:flutter/material.dart';
import 'package:simple_animations/simple_animations.dart';

class GenericFadeTransition extends StatelessWidget {
  final Duration delay;
  final Duration duration;
  final Widget child;
  final double yStart;
  final double yEnd;
  final double xStart;
  final double xEnd;
  final Curve animationCurve;
  final void Function()? onStart;
  final void Function()? onComplete;

  GenericFadeTransition({
    required this.child,
    this.delay = const Duration(),
    this.duration = const Duration(seconds: 1),
    this.xEnd = 0,
    this.xStart = 0,
    this.yStart = 100,
    this.yEnd = 0,
    this.animationCurve = Curves.ease,
    this.onStart,
    this.onComplete,
  });

  @override
  Widget build(BuildContext context) {
    final _tween = TimelineTween<_AniProps>(curve: animationCurve)
      ..addScene(begin: delay, duration: duration).animate(
        _AniProps.y,
        tween: Tween(
          begin: yStart,
          end: yEnd,
        ),
      )
      ..addScene(begin: delay, duration: duration).animate(
        _AniProps.x,
        tween: Tween(
          begin: xStart,
          end: xEnd,
        ),
      )
      ..addScene(begin: delay, duration: duration).animate(
        _AniProps.opacity,
        tween: Tween(
          begin: 0.0,
          end: 1.0,
        ),
      );

    return PlayAnimation<TimelineValue<_AniProps>>(
      tween: _tween,
      duration: duration,
      curve: animationCurve,
      child: child,
      delay: delay,
      onComplete: onComplete,
      onStart: onStart,
      builder: (context, animationChild, value) {
        return Transform.translate(
          offset: Offset(
            value.get(_AniProps.x),
            value.get(_AniProps.y),
          ),
          child: Opacity(
            opacity: value.get(_AniProps.opacity),
            child: animationChild,
          ),
        );
      },
    );
  }
}

enum _AniProps { x, y, opacity }
